/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension.base;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

/**
 * JFace {@link Action}, which uses an {@link EObject} as target.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 4C545F6468D6051419AA91C0E3774061
 */
public abstract class EObjectActionBase<T extends EObject> extends Action {

	/** Factory for the action instances used for this menu */
	public static interface EObjectActionFactory<T extends EObject> {
		/** Creates and returns an action instance for the given target. */
		EObjectActionBase<T> createAction(T target);
	}

	/** Stores the target element. */
	private T target;

	/** Constructor. */
	public EObjectActionBase(String text, ImageDescriptor image) {
		super(text, image);
	}

	/** Constructor. */
	public EObjectActionBase(String text) {
		this(text, null);
	}

	/** Constructor. */
	public EObjectActionBase(T target, String text, ImageDescriptor image) {
		this(text, image);
		setTarget(target);
	}

	/** Constructor. */
	public EObjectActionBase(T target, String text) {
		this(target, text, null);
	}

	/** Sets the target object. */
	public void setTarget(T target) {
		this.target = target;
	}

	/** Returns the current target. */
	public T getTarget() {
		return target;
	}

	/** Refreshes the action's target and enabled state. */
	// TODO @review DT: should a user take setTarget or refresh for a new
	// element?
	// Should a constructor take setTarget or refresh? should there be a refresh
	// method for the currently set element, i. e. refresh(). Possibility:
	// refresh() for current element and refresh(element){setTarget(element);
	// refresh()}.
	public final void refresh(T element) {
		setTarget(element);
		if (element == null) {
			setEnabled(false);
			return;
		}
		setEnabled(computeEnabled());
	}

	/**
	 * Computes the enabled state of the action for the current target, which is
	 * not <code>null</code> when this method is called. The default
	 * implementation returns <code>true</code>.
	 */
	protected boolean computeEnabled() {
		return true;
	}
}
