/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.views;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.fortiss.tooling.kernel.ui.extension.IModelElementHandler;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;
import org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils;

/**
 * Double-click listener for navigator and marker view.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 8E97B6DDA60D757813BABD53879EE92A
 */
final class DoubleClick implements IDoubleClickListener {

	/** Stores the use-raw-eobject flag. */
	private final boolean useRawEObject;

	/** Constructor. */
	public DoubleClick() {
		this(false);
	}

	/** Constructor. */
	public DoubleClick(boolean useRawEObject) {
		this.useRawEObject = useRawEObject;
	}

	/** {@inheritDoc} */
	@Override
	public void doubleClick(DoubleClickEvent event) {
		// delegate to the editor service
		if (event.getSelection() instanceof IStructuredSelection) {
			EObject element = EObjectSelectionUtils.getFirstElement(event
					.getSelection());
			if (element != null) {
				if (!useRawEObject) {
					IModelElementHandler<EObject> handler = IModelElementHandlerService.INSTANCE
							.getModelElementHandler(element);
					if (handler != null) {
						element = handler
								.handleOpenModelElementRequest(element);
					}
				}
				IModelEditorBindingService.INSTANCE.openInEditor(element);
			}
		}
	}
}
